from argparse import Namespace

import pytest

from tuxrun.argparse import filter_options, setup_parser


def test_filter_options():
    assert filter_options(Namespace()) == {}
    assert filter_options(Namespace(hello="world")) == {"hello": "world"}
    assert filter_options(Namespace(hello="world", debug=True)) == {"hello": "world"}


def test_timeouts_parser():
    assert setup_parser().parse_args(["--timeouts", "boot=1"]).timeouts == {"boot": 1}
    assert setup_parser().parse_args(
        ["--timeouts", "boot=1", "deploy=42"]
    ).timeouts == {"boot": 1, "deploy": 42}

    with pytest.raises(SystemExit):
        setup_parser().parse_args(["--timeouts", "boot=a"])

    with pytest.raises(SystemExit):
        setup_parser().parse_args(["--timeouts", "booting=1"])



import subprocess
import time
from hashlib import sha1
from pathlib import Path

import pytest

from tuxrun.assets import get_rootfs
from tuxrun.devices import Device

seven_hours = 7 * 60 * 60


def rewind_timestamp(filename, interval):
    past = time.time() - interval
    subprocess.check_call(["touch", "-d", f"@{past}", str(filename)])


class TestGetRootfs:
    def test_local_file(self):
        assert (
            get_rootfs(Device.select("qemu-arm64"), "/path/to/file") == "/path/to/file"
        )

    def test_default(self):
        rootfs = get_rootfs(Device.select("qemu-arm64"))
        assert Path(rootfs).exists()

    def test_downloads_remote_file(self, get, mocker):
        rootfs = get_rootfs(
            Device.select("qemu-arm64"), "https://example.com/rootfs.img"
        )
        assert Path(rootfs).exists()
        get.assert_called()
        assert get.call_args[0][0] == "https://example.com/rootfs.img"

    def test_caches_downloads(self, get):
        get_rootfs(Device.select("qemu-arm64"))
        get_rootfs(Device.select("qemu-arm64"))
        assert get.call_count == 1

    def test_writes_data_to_disk(self, response):
        response.iter_content.return_value = [b"123"]
        rootfs = get_rootfs(Device.select("qemu-x86_64"))
        assert Path(rootfs).read_text() == "123"

    @pytest.fixture
    def build_response(self, mocker):
        def __build_response__(data):
            r = mocker.MagicMock()
            r.iter_content.return_value = [data]
            data_hash = sha1(data).hexdigest()
            r.headers = {"ETag": f'"{data_hash}"', "Content-Length": str(len(data))}
            return r

        return __build_response__

    def test_update_after_6h_if_etag_changed(self, get, build_response):
        response1 = build_response(b"123")
        response2 = build_response(b"456")
        get.side_effect = [response1, response2]

        rootfs = get_rootfs(Device.select("qemu-x86_64"))
        rewind_timestamp(rootfs, seven_hours)

        rootfs = get_rootfs(Device.select("qemu-x86_64"))
        assert Path(rootfs).read_text() == "456"

    def test_uses_cache_after_6h_with_same_etag(self, response):
        rootfs = get_rootfs(Device.select("qemu-x86_64"))
        rewind_timestamp(rootfs, seven_hours)
        response.iter_content.assert_called()

        response.reset_mock()
        get_rootfs(Device.select("qemu-x86_64"))
        response.iter_content.assert_not_called()

    def test_uses_cached_version_when_download_fails(self, get, response):
        get.side_effect = [response, RuntimeError("BOOM")]
        r1 = get_rootfs(Device.select("qemu-x86_64"))
        rewind_timestamp(r1, seven_hours)
        r2 = get_rootfs(Device.select("qemu-x86_64"))
        assert r1 == r2

    def test_pass_without_cache_when_download_fails(self, get, response):
        get.side_effect = RuntimeError("BOOM")
        assert (
            get_rootfs(Device.select("qemu-x86_64"))
            == "https://storage.tuxboot.com/buildroot/x86_64/rootfs.ext4.zst"
        )


if __name__ == "__main__":
    rootfs = get_rootfs(Device.select("qemu-x86_64"))
    print(rootfs)



# vim: set ts=4
#
# Copyright 2024-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun.tests import Test


class XfsTests(Test):
    devices = ["qemu-arm64", "qemu-x86_64", "fvp-aemva"]
    configfile: str = ""
    timeout = 90
    need_test_definition = True

    def render(self, **kwargs):
        kwargs["name"] = self.name
        kwargs["test_filesystem"] = self.test_filesystem
        kwargs["timeout"] = self.timeout
        return self._render("xfstests.yaml.jinja2", **kwargs)


class XfsTestsBtrfs(XfsTests):
    name = "xfstests-btrfs"
    test_filesystem = "btrfs"


class XfsTestsExt4(XfsTests):
    name = "xfstests-ext4"
    test_filesystem = "ext4"


class XfsTestsF2fs(XfsTests):
    name = "xfstests-f2fs"
    test_filesystem = "f2fs"


class XfsTestsNilfs2(XfsTests):
    name = "xfstests-nilfs2"
    test_filesystem = "nilfs2"


class XfsTestsXfs(XfsTests):
    name = "xfstests-xfs"
    test_filesystem = "xfs"



# vim: set ts=4
#
# Copyright 2024-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun.tests import Test


class HackingSession(Test):
    devices = [
        "qemu-armv6",
        "qemu-armv7",
        "qemu-arm64",
        "qemu-i386",
        "qemu-x86_64",
        "fvp-aemva",
    ]
    name = "hacking-session"
    timeout = 20
    need_test_definition = True

    def render(self, **kwargs):
        kwargs["name"] = self.name
        kwargs["timeout"] = self.timeout

        return self._render("hackingsession.yaml.jinja2", **kwargs)



#!/usr/bin/python3
import argparse
import fnmatch
import shutil
import subprocess
import sys
import tempfile
from pathlib import Path
from typing import Any, Dict

from tuxrun.devices import Device
from tuxrun.tests import Test
from tuxrun.yaml import yaml_load


###########
# Helpers #
###########
def get_results(tmpdir: Path) -> Dict:
    required_keys = {"msg", "lvl", "dt"}
    res: Dict[Any, Any] = {}
    data = yaml_load((tmpdir / "logs.yaml").read_text(encoding="utf-8"))

    if data is None:
        return {}

    for line in data:
        if not isinstance(line, dict):
            continue
        if not required_keys.issubset(set(line.keys())):
            continue
        if line["lvl"] != "results":
            continue
        definition = line["msg"]["definition"]
        case = line["msg"]["case"]
        del line["msg"]["definition"]
        del line["msg"]["case"]
        res.setdefault(definition, {})[case] = line["msg"]

    return res


def get_simple_results(res: Dict) -> Dict:
    results = {
        "boot": res.get("lava", {}).get("login-action", {}).get("result", "fail")
    }
    for name in res:
        if name == "lava":
            continue
        key = "_".join(name.split("_")[1:])
        if all(res[name][case]["result"] in ["skip", "pass"] for case in res[name]):
            results[key] = "pass"
        else:
            results[key] = "fail"
    return results


def get_job_result(results: Dict, simple_results: Dict) -> str:
    # lava.job is missing: error
    lava = results.get("lava", {}).get("job")
    if lava is None:
        return "error"

    if lava["result"] == "fail":
        if lava.get("error_type") == "Job":
            return "fail"
        return "error"

    if all(v == "pass" for (k, v) in simple_results.items()):
        return "pass"
    return "fail"


def run(device, test, runtime, debug):
    tmpdir = Path(tempfile.mkdtemp(prefix="tuxrun-"))

    args = [
        "python3",
        "-m",
        "tuxrun",
        "--device",
        device,
        "--runtime",
        runtime,
        "--log-file-yaml",
        str(tmpdir / "logs.yaml"),
        "--log-file",
        "/dev/null",
    ]
    if test:
        args.extend(["--tests", test])

    if device == "fvp-morello-android":
        args.extend(
            [
                "--mcp-fw",
                "https://storage.tuxboot.com/fvp-morello-android/mcp_fw.bin",
                "--mcp-romfw",
                "https://storage.tuxboot.com/fvp-morello-android/mcp_romfw.bin",
                "--rootfs",
                "https://storage.tuxboot.com/fvp-morello-android/android-nano.img.xz",
                "--scp-fw",
                "https://storage.tuxboot.com/fvp-morello-android/scp_fw.bin",
                "--scp-romfw",
                "https://storage.tuxboot.com/fvp-morello-android/scp_romfw.bin",
                "--fip",
                "https://storage.tuxboot.com/fvp-morello-busybox/fip.bin",
            ]
        )
        if test == "binder":
            args.extend(
                [
                    "--parameters",
                    "USERDATA=https://storage.tuxboot.com/fvp-morello-android/userdata.tar.xz",
                ]
            )
        elif test == "bionic":
            args.extend(
                [
                    "--parameters",
                    "USERDATA=https://storage.tuxboot.com/fvp-morello-android/userdata.tar.xz",
                ]
            )
        elif test == "boringssl":
            args.extend(
                [
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
        elif test == "compartment":
            args.extend(
                [
                    "--parameters",
                    "USERDATA=https://storage.tuxboot.com/fvp-morello-android/userdata.tar.xz",
                ]
            )
        elif test == "libjpeg-turbo":
            args.extend(
                [
                    "--parameters",
                    "LIBJPEG_TURBO_URL=https://storage.tuxboot.com/fvp-morello-android/libjpeg-turbo.tar.xz",
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
        elif test == "libpng":
            args.extend(
                [
                    "--parameters",
                    "PNG_URL=https://storage.tuxboot.com/fvp-morello-android/png-testfiles.tar.xz",
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
        elif test == "libpdfium":
            args.extend(
                [
                    "--parameters",
                    "PDFIUM_URL=https://storage.tuxboot.com/fvp-morello-android/pdfium-testfiles.tar.xz",
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
        elif test == "lldb":
            args.extend(
                [
                    "--parameters",
                    "LLDB_URL=https://storage.tuxboot.com/fvp-morello-android/lldb_tests.tar.xz",
                    "--parameters",
                    "TC_URL=https://storage.tuxboot.com/fvp-morello-android/morello-clang.tar.xz",
                ]
            )
        elif test == "logd":
            args.extend(
                [
                    "--parameters",
                    "USERDATA=https://storage.tuxboot.com/fvp-morello-android/userdata.tar.xz",
                ]
            )
        elif test == "zlib":
            args.extend(
                [
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
    elif device == "fvp-morello-busybox":
        args.extend(
            [
                "--mcp-fw",
                "https://storage.tuxboot.com/fvp-morello-busybox/mcp_fw.bin",
                "--mcp-romfw",
                "https://storage.tuxboot.com/fvp-morello-busybox/mcp_romfw.bin",
                "--rootfs",
                "https://storage.tuxboot.com/fvp-morello-busybox/busybox.img.xz",
                "--scp-fw",
                "https://storage.tuxboot.com/fvp-morello-busybox/scp_fw.bin",
                "--scp-romfw",
                "https://storage.tuxboot.com/fvp-morello-busybox/scp_romfw.bin",
                "--fip",
                "https://storage.tuxboot.com/fvp-morello-busybox/fip.bin",
            ]
        )
    elif device == "fvp-morello-ubuntu":
        args.extend(
            [
                "--mcp-fw",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/mcp_fw.bin",
                "--mcp-romfw",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/mcp_romfw.bin",
                "--scp-fw",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/scp_fw.bin",
                "--scp-romfw",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/scp_romfw.bin",
                "--fip",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/fip.bin",
            ]
        )

    try:
        ret = subprocess.call(args)
        if ret != 0:
            print(f"Command return non-zero exist status {ret}")
            print((tmpdir / "logs.yaml").read_text(encoding="utf-8"))
            return ret

        results = get_results(tmpdir)
        simple_results = get_simple_results(results)
        result = get_job_result(results, simple_results)

        if debug:
            print("Results:")
            for res in results:
                print(f"* {res}: {results[res]}")

            print("\nSimple results:")
            for res in simple_results:
                print(f"* {res}: {simple_results[res]}")
            print(f"Result {result}")
        else:
            print(f"{result}")
        assert result == "pass"
    finally:
        shutil.rmtree(tmpdir)


##############
# Entrypoint #
##############
def main():
    parser = argparse.ArgumentParser(description="Integration tests")
    parser.add_argument("--devices", nargs="+", required=True, help="device")
    parser.add_argument(
        "--tests",
        default=["boot"],
        nargs="+",
        help="tests",
    )
    parser.add_argument("--debug", default=False, action="store_true", help="debug")
    parser.add_argument(
        "--runtime",
        default="podman",
        choices=["docker", "null", "podman"],
        help="Runtime",
    )
    options = parser.parse_args()

    if len(options.devices) == 1 and "*" in options.devices[0]:
        pat = options.devices[0]
        options.devices = [
            d.name for d in Device.list() if fnmatch.fnmatch(d.name, pat)
        ]
        for qemu in ["qemu-riscv32", "qemu-sh4"]:
            if qemu in options.devices:
                options.devices.remove(qemu)

    for device in options.devices:
        tests = options.tests.copy()
        if tests == ["all"]:
            tests = ["boot"] + Test.list(device=device)
        for test in tests:
            print(f"=> {device} x {test}")
            if run(
                device, "" if test == "boot" else test, options.runtime, options.debug
            ):
                return 1
            print("")


if __name__ == "__main__":
    sys.exit(main())



# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import contextlib
import json
from pathlib import Path

from tuxrun.requests import requests_get


class InvalidTuxBuild(Exception):
    pass


class TuxBuild:
    Invalid = InvalidTuxBuild

    def __init__(self):
        self.kernel = None
        self.modules = []
        self.target_arch = None

    def parse(self, url, data):
        try:
            metadata = json.loads(data)
        except json.JSONDecodeError as e:
            raise self.Invalid(f"Invalid metadata.json: {e}")

        try:
            self.target_arch = metadata["build"]["target_arch"]
        except KeyError:
            raise self.Invalid("{url}/metadata.json is invalid")

        with contextlib.suppress(IndexError, KeyError):
            self.kernel = url + "/" + metadata["results"]["artifacts"]["kernel"][0]
        with contextlib.suppress(IndexError, KeyError):
            self.modules = [
                url + "/" + metadata["results"]["artifacts"]["modules"][0],
                "/",
            ]

        if self.kernel is None:
            raise self.Invalid("Missing kernel in directory")


class TuxBuildBuild(TuxBuild):
    def __init__(self, url):
        super().__init__()

        self.url = url
        ret = requests_get(f"{url}/metadata.json")
        if ret.status_code != 200:
            raise self.Invalid(f"{url}/metadata.json is missing")

        self.parse(url, ret.text)


class TuxMakeBuild(TuxBuild):
    def __init__(self, directory):
        super().__init__()

        self.location = Path(directory).resolve()
        self.url = f"file://{self.location}"
        metadata_file = self.location / "metadata.json"
        if not self.location.is_dir():
            raise self.Invalid(f"{directory} is not a directory")
        if not metadata_file.exists():
            raise self.Invalid(
                f"{directory} is not a valid TuxMake artifacts directory: missing metadata.json"
            )

        self.parse(f"file://{self.location}", metadata_file.read_text(encoding="utf-8"))



# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import argparse
import os
import re
import sys
from abc import ABC, abstractmethod
from io import StringIO
from pathlib import Path
from urllib.parse import urlparse

from ruamel.yaml import YAML

from tuxrun import requests, xdg
from tuxrun.yaml import yaml_load


class ProgressIndicator(ABC):
    @abstractmethod
    def progress(self, percent):
        """
        This method should display the current percentage to the user
        """

    @abstractmethod
    def finish(self):
        """
        This method should display to the user that the process has finished
        """

    @classmethod
    def get(cls, name: str) -> "ProgressIndicator":
        if sys.stderr.isatty():
            return TTYProgressIndicator(name)
        else:
            return NoProgressIndicator()


class NoProgressIndicator(ProgressIndicator):
    def progress(self, percent):
        pass

    def finish(self):
        pass


class TTYProgressIndicator(ProgressIndicator):
    def __init__(self, name):
        self.name = name

    def progress(self, percent: int) -> None:
        sys.stderr.write(f"\r{self.name} ... %3d%%" % percent)

    def finish(self) -> None:
        sys.stderr.write("\n")


COMPRESSIONS = {
    ".tar.xz": ("tar", "xz"),
    ".tar.gz": ("tar", "gz"),
    ".tgz": ("tar", "gz"),
    ".gz": (None, "gz"),
    ".xz": (None, "xz"),
    ".zst": (None, "zstd"),
    ".py": ("file", None),
    ".sh": ("file", None),
}


def compression(path):
    for ext, ret in COMPRESSIONS.items():
        if path.endswith(ext):
            return ret
    return (None, None)


def notnone(value, fallback):
    if value is None:
        return fallback
    return value


def get_new_output_dir(cache_dir):
    base = xdg.get_cache_dir() / "tests"
    if cache_dir:
        base = Path(f"{os.path.abspath(cache_dir)}/tests")
    base.mkdir(parents=True, exist_ok=True)
    existing = [int(f.name) for f in base.glob("[0-9]*")]
    if existing:
        new = max(existing) + 1
    else:
        new = 1
    while True:
        new_dir = base / str(new)
        try:
            new_dir.mkdir()
            break
        except FileExistsError:
            new += 1
    return new_dir


def slugify(s):
    s = s.lower().strip()
    s = re.sub(r"[^\w\s-]", "", s)
    s = re.sub(r"[\s_-]+", "-", s)
    s = re.sub(r"^-+|-+$", "", s)
    return s


def pathurlnone(string):
    if string is None:
        return None
    url = urlparse(string)
    if url.scheme in ["http", "https"]:
        return string
    if url.scheme not in ["", "file"]:
        raise argparse.ArgumentTypeError(f"Invalid scheme '{url.scheme}'")

    path = Path(string if url.scheme == "" else url.path)
    if not path.exists():
        raise argparse.ArgumentTypeError(f"{path} no such file or directory")
    return f"file://{path.expanduser().resolve()}"


def pathnone(string):
    if string is None:
        return None

    path = Path(string)
    if not path.exists():
        raise argparse.ArgumentTypeError(f"{path} no such file or directory")
    return path.expanduser().resolve()


def callback(dataset=None, header=None, method=None, token=None, url=None, **kwargs):
    if dataset and header and method and token and url:
        s = requests.get_session(retries=3)
        s.headers.update({header: token})
        if method == "POST":
            s.post(url=url)
        if method == "GET":
            s.get(url=url)


def notify(notify):
    if not notify:
        return
    if "callbacks" in notify:
        for cb in notify.get("callbacks"):
            callback(**cb)


def mask_secrets(jobdef: str) -> str:
    data = yaml_load(jobdef)

    if "secrets" in data:
        for secret in data["secrets"]:
            data["secrets"][secret] = "********"

        yaml = YAML(typ="rt")
        yaml.preserve_quotes = True  # type: ignore
        yaml_stream = StringIO()

        yaml.dump(data, yaml_stream)
        masked_jobdef = yaml_stream.getvalue()
        return masked_jobdef

    return jobdef



# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import yaml

FullLoader = None
for loader in ["CFullLoader", "CLoader", "FullLoader", "Loader"]:
    if hasattr(yaml, loader):
        if loader != "CFullLoader":
            print("Warning: using python yaml loader")
        FullLoader = getattr(yaml, loader)
        break

assert FullLoader is not None


def yaml_load(data):
    return yaml.load(data, Loader=FullLoader)



# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import contextlib
import logging
import os
import signal
import subprocess
import time
from pathlib import Path

from tuxrun.templates import wrappers

BASE = (Path(__file__) / "..").resolve()
LOG = logging.getLogger("tuxrun")


class Runtime:
    binary = ""
    container = False
    prefix = [""]

    def __init__(self):
        self.__bindings__ = []
        self.__image__ = None
        self.__name__ = None
        self.__pre_proc__ = None
        self.__proc__ = None
        self.__sub_procs__ = []
        self.__ret__ = None

    @classmethod
    def select(cls, name):
        if name == "docker":
            return DockerRuntime
        if name == "podman":
            return PodmanRuntime
        return NullRuntime

    def bind(self, src, dst=None, ro=False, device=False):
        if dst is None:
            dst = src
        self.__bindings__.append((str(src), dst, ro, device))

    def image(self, image):
        self.__image__ = image

    def name(self, name):
        self.__name__ = name

    def pre_run(self, tmpdir):
        pass

    def post_run(self):
        pass

    def cmd(self, args):
        raise NotImplementedError()  # pragma: no cover

    @contextlib.contextmanager
    def run(self, args):
        args = self.cmd(args)
        LOG.debug("Calling %s", " ".join(args))
        try:
            self.__proc__ = subprocess.Popen(
                args,
                bufsize=1,
                stderr=subprocess.PIPE,
                text=True,
                preexec_fn=os.setpgrp,
            )
            yield
        except FileNotFoundError as exc:
            LOG.error("File not found '%s'", exc.filename)
            raise
        except Exception as exc:
            LOG.exception(exc)
            if self.__proc__ is not None:
                self.kill()
                _, errs = self.__proc__.communicate()
                for err in [e for e in errs.split("\n") if e]:
                    LOG.error("err: %s", err)
            raise
        finally:
            if self.__proc__ is not None:
                self.__ret__ = self.__proc__.wait()
            for proc in self.__sub_procs__:
                proc.wait()

    def lines(self):
        return self.__proc__.stderr

    def kill(self):
        if self.__proc__:
            self.__proc__.send_signal(signal.SIGTERM)

    def ret(self):
        return self.__ret__


class ContainerRuntime(Runtime):
    bind_guestfs = True
    container = True

    def __init__(self):
        super().__init__()
        self.bind("/boot", ro=True)
        self.bind("/lib/modules", ro=True)
        # Bind /dev/kvm is available
        if Path("/dev/kvm").exists():
            self.bind("/dev/kvm", device=True)
        # Create /var/tmp/.guestfs-$id
        if self.bind_guestfs:
            guestfs = Path(f"/var/tmp/.guestfs-{os.getuid()}")
            guestfs.mkdir(exist_ok=True)
            self.bind(guestfs, "/var/tmp/.guestfs-0")

    def cmd(self, args):
        prefix = self.prefix.copy()
        srcs = set()
        dsts = set()
        for binding in self.__bindings__:
            (src, dst, ro, device) = binding
            if src in srcs:
                LOG.error("Duplicated mount source %r", src)
                raise Exception("Duplicated mount source %r" % src)
            if dst in dsts:
                LOG.error("Duplicated mount destination %r", dst)
                raise Exception("Duplicated mount destination %r" % dst)
            srcs.add(src)
            dsts.add(dst)
            ro = "ro" if ro else "rw"
            prefix.extend(["--device" if device else "-v", f"{src}:{dst}:{ro}"])
        prefix.extend(["--name", self.__name__])
        return prefix + [self.__image__] + args

    def kill(self):
        args = [self.binary, "stop", "--time", "60", self.__name__]
        with contextlib.suppress(FileNotFoundError):
            proc = subprocess.Popen(
                args,
                stderr=subprocess.DEVNULL,
                stdout=subprocess.DEVNULL,
                preexec_fn=os.setpgrp,
            )
            self.__sub_procs__.append(proc)


class DockerRuntime(ContainerRuntime):
    # Do not bind or libguestfs will fail at runtime
    # "security: cached appliance /var/tmp/.guestfs-0 is not owned by UID 0"
    bind_guestfs = False
    binary = "docker"
    prefix = ["docker", "run", "--rm", "--hostname", "tuxrun"]

    def pre_run(self, tmpdir):
        # Render and bind the docker wrapper
        wrap = (
            wrappers()
            .get_template("docker.jinja2")
            .render(runtime="docker", volume=str(tmpdir / "dispatcher" / "tmp"))
        )
        LOG.debug("docker wrapper")
        LOG.debug(wrap)
        (tmpdir / "docker").write_text(wrap, encoding="utf-8")
        (tmpdir / "docker").chmod(0o755)
        self.bind(str(tmpdir / "docker"), "/usr/local/bin/docker", True)

        # Bind the docker socket
        self.bind("/var/run/docker.sock")


class PodmanRuntime(ContainerRuntime):
    binary = "podman"
    prefix = ["podman", "run", "--log-driver=none", "--rm", "--hostname", "tuxrun"]
    network = None

    def pre_run(self, tmpdir):
        # Render and bind the docker wrapper
        self.network = os.path.basename(tmpdir)
        self.prefix.extend(["--network", self.network])
        subprocess.run(["podman", "network", "create", self.network])
        wrap = (
            wrappers()
            .get_template("docker.jinja2")
            .render(
                runtime="podman",
                volume=str(tmpdir / "dispatcher" / "tmp"),
                network=self.network,
            )
        )
        LOG.debug("docker wrapper")
        LOG.debug(wrap)
        (tmpdir / "docker").write_text(wrap, encoding="utf-8")
        (tmpdir / "docker").chmod(0o755)
        self.bind(str(tmpdir / "docker"), "/usr/local/bin/docker", True)

        # Start podman system service and bind the socket
        socket = tmpdir / "podman.sock"
        self.bind(socket, "/run/podman/podman.sock")

        args = [
            self.binary,
            "system",
            "service",
            "--time",
            "0",
            f"unix://{socket}",
        ]
        self.__pre_proc__ = subprocess.Popen(
            args,
            stderr=subprocess.DEVNULL,
            stdout=subprocess.DEVNULL,
            preexec_fn=os.setpgrp,
        )
        # wait for the socket
        for _ in range(60):
            if socket.exists():
                return
            time.sleep(1)
        raise Exception(f"Unable to create podman socket at {socket}")

    def post_run(self):
        if self.network:
            subprocess.run(["podman", "network", "rm", self.network])
        if self.__pre_proc__ is None:
            return
        self.__pre_proc__.kill()
        self.__pre_proc__.wait()


class NullRuntime(Runtime):
    def cmd(self, args):
        return args
